<?php

declare(strict_types=1);

namespace SimAVRPHP;

use SimAVRPHP\Exceptions\InvalidAddressException;
use Psr\Log\LoggerInterface;

class AVR
{
    public ProgramCounter $pc;

    public Registers $registers;

    /**
     * @var Stack Contains the CPU stack
     */
    public Stack $stack;

    public string $memory = "";

    public function __construct(public LoggerInterface $logger)
    {
        $this->pc = new ProgramCounter();
        $this->registers = new Registers();
        $this->stack = new Stack();
        $this->reset();
    }

    public function setMemory(string $memory): void
    {
        $this->memory = $memory;
    }

    public function reset(): void
    {
        $this->pc->reset();
    }

    public function step(): void
    {
        $rawOperation = $this->getOperation();
        $operation = Opcode\Instruction::createInstructionFromOpcode($this, $rawOperation);

        $this->logger->debug("Prepare instruction for execution", ['PC' => $this->pc->get(), 'Class' => $operation::class]);
        $operation->execute();
    }

    private function getOperation(): string
    {
        $opcode = unpack("v2", $this->memory, $this->pc->get());
        if ($opcode == FALSE) {
            throw new InvalidAddressException();
        }

        return str_pad(decbin((int)$opcode[1]), 16, "0", STR_PAD_LEFT) . str_pad(decbin((int)$opcode[2]), 16, "0", STR_PAD_LEFT);
    }
}
